@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "qztools.h"
-#include "mainapplication.h"
+#include "datapaths.h"
 #include "settings.h"
 
 #include <QTextDocument>
@@ -39,6 +39,7 @@
 #include <QSysInfo>
 #include <QProcess>
 #include <QMessageBox>
+#include <QWebFrame>
 
 #if QT_VERSION >= 0x050000
 #include <QUrlQuery>
@@ -81,15 +82,21 @@ QPixmap QzTools::pixmapFromByteArray(const QByteArray &data)
 }
 
 QString QzTools::readAllFileContents(const QString &filename)
+{
+    return QString::fromUtf8(readAllFileByteContents(filename));
+}
+
+QByteArray QzTools::readAllFileByteContents(const QString &filename)
 {
     QFile file(filename);
-    if (file.open(QFile::ReadOnly)) {
-        QString a = QString::fromUtf8(file.readAll());
+
+    if (!filename.isEmpty() && file.open(QFile::ReadOnly)) {
+        const QByteArray a = file.readAll();
         file.close();
         return a;
     }
 
-    return QString();
+    return QByteArray();
 }
 
 void QzTools::centerWidgetOnScreen(QWidget* w)
@@ -189,6 +196,31 @@ QString QzTools::urlEncodeQueryString(const QUrl &url)
     return returnString;
 }
 
+QString QzTools::fromPunycode(const QString &str)
+{
+    if (!str.startsWith(QL1S("xn--")))
+        return str;
+
+    // QUrl::fromAce will only decode domains from idn whitelist
+    const QString decoded = QUrl::fromAce(str.toUtf8() + QByteArray(".org"));
+    return decoded.left(decoded.size() - 4);
+}
+
+QString QzTools::escapeSqlString(QString urlString)
+{
+    const static QString &escapeString = QL1S("!");
+    urlString.replace(escapeString, escapeString + escapeString);
+    urlString.replace(QL1S("_"), escapeString + QL1S("_"));
+    urlString.replace(QL1S("%"), escapeString + QL1S("%"));
+
+    return urlString;
+}
+
+QUrl QzTools::frameUrl(QWebFrame* frame)
+{
+    return frame->url().isEmpty() ? frame->baseUrl() : frame->url();
+}
+
 QString QzTools::ensureUniqueFilename(const QString &name, const QString &appendFormat)
 {
     if (!QFile::exists(name)) {
@@ -378,6 +410,14 @@ QString QzTools::applyDirectionToPage(QString &pageContents)
     return pageContents;
 }
 
+QString QzTools::truncatedText(const QString &text, int size)
+{
+    if (text.length() > size) {
+        return text.left(size) + QL1S("..");
+    }
+    return text;
+}
+
 // Thanks to http://www.qtcentre.org/threads/3205-Toplevel-widget-with-rounded-corners?p=17492#post17492
 QRegion QzTools::roundedRect(const QRect &rect, int radius)
 {
@@ -416,7 +456,7 @@ QIcon QzTools::iconFromFileName(const QString &fileName)
     }
 
     QFileIconProvider iconProvider;
-    QTemporaryFile tempFile(mApp->tempPath() + "/XXXXXX." + tempInfo.suffix());
+    QTemporaryFile tempFile(DataPaths::path(DataPaths::Temp) + "/XXXXXX." + tempInfo.suffix());
     tempFile.open();
     tempInfo.setFile(tempFile.fileName());
 
@@ -456,65 +496,65 @@ bool QzTools::isUtf8(const char* string)
     const unsigned char* bytes = (const unsigned char*)string;
     while (*bytes) {
         if ((// ASCII
-                    bytes[0] == 0x09 ||
-                    bytes[0] == 0x0A ||
-                    bytes[0] == 0x0D ||
-                    (0x20 <= bytes[0] && bytes[0] <= 0x7F)
-                )
+                bytes[0] == 0x09 ||
+                bytes[0] == 0x0A ||
+                bytes[0] == 0x0D ||
+                (0x20 <= bytes[0] && bytes[0] <= 0x7F)
+            )
            ) {
             bytes += 1;
             continue;
         }
 
         if ((// non-overlong 2-byte
-                    (0xC2 <= bytes[0] && bytes[0] <= 0xDF) &&
-                    (0x80 <= bytes[1] && bytes[1] <= 0xBF)
-                )
+                (0xC2 <= bytes[0] && bytes[0] <= 0xDF) &&
+                (0x80 <= bytes[1] && bytes[1] <= 0xBF)
+            )
            ) {
             bytes += 2;
             continue;
         }
 
         if ((// excluding overlongs
-                    bytes[0] == 0xE0 &&
-                    (0xA0 <= bytes[1] && bytes[1] <= 0xBF) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF)
-                ) ||
-                (// straight 3-byte
-                    ((0xE1 <= bytes[0] && bytes[0] <= 0xEC) ||
-                     bytes[0] == 0xEE ||
-                     bytes[0] == 0xEF) &&
-                    (0x80 <= bytes[1] && bytes[1] <= 0xBF) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF)
-                ) ||
-                (// excluding surrogates
-                    bytes[0] == 0xED &&
-                    (0x80 <= bytes[1] && bytes[1] <= 0x9F) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF)
-                )
+                bytes[0] == 0xE0 &&
+                (0xA0 <= bytes[1] && bytes[1] <= 0xBF) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF)
+            ) ||
+            (// straight 3-byte
+                ((0xE1 <= bytes[0] && bytes[0] <= 0xEC) ||
+                 bytes[0] == 0xEE ||
+                 bytes[0] == 0xEF) &&
+                (0x80 <= bytes[1] && bytes[1] <= 0xBF) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF)
+            ) ||
+            (// excluding surrogates
+                bytes[0] == 0xED &&
+                (0x80 <= bytes[1] && bytes[1] <= 0x9F) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF)
+            )
            ) {
             bytes += 3;
             continue;
         }
 
         if ((// planes 1-3
-                    bytes[0] == 0xF0 &&
-                    (0x90 <= bytes[1] && bytes[1] <= 0xBF) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
-                    (0x80 <= bytes[3] && bytes[3] <= 0xBF)
-                ) ||
-                (// planes 4-15
-                    (0xF1 <= bytes[0] && bytes[0] <= 0xF3) &&
-                    (0x80 <= bytes[1] && bytes[1] <= 0xBF) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
-                    (0x80 <= bytes[3] && bytes[3] <= 0xBF)
-                ) ||
-                (// plane 16
-                    bytes[0] == 0xF4 &&
-                    (0x80 <= bytes[1] && bytes[1] <= 0x8F) &&
-                    (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
-                    (0x80 <= bytes[3] && bytes[3] <= 0xBF)
-                )
+                bytes[0] == 0xF0 &&
+                (0x90 <= bytes[1] && bytes[1] <= 0xBF) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
+                (0x80 <= bytes[3] && bytes[3] <= 0xBF)
+            ) ||
+            (// planes 4-15
+                (0xF1 <= bytes[0] && bytes[0] <= 0xF3) &&
+                (0x80 <= bytes[1] && bytes[1] <= 0xBF) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
+                (0x80 <= bytes[3] && bytes[3] <= 0xBF)
+            ) ||
+            (// plane 16
+                bytes[0] == 0xF4 &&
+                (0x80 <= bytes[1] && bytes[1] <= 0x8F) &&
+                (0x80 <= bytes[2] && bytes[2] <= 0xBF) &&
+                (0x80 <= bytes[3] && bytes[3] <= 0xBF)
+            )
            ) {
             bytes += 4;
             continue;
@@ -781,19 +821,21 @@ QString QzTools::escape(const QString &string)
 #endif
 }
 
-#ifdef QZ_WS_X11
 void* QzTools::X11Display(const QWidget* widget)
 {
     Q_UNUSED(widget)
 
+#ifdef QZ_WS_X11
 #if QT_VERSION >= 0x050000
     return qApp->platformNativeInterface()->nativeResourceForWindow("display", widget->windowHandle());
 #else
     return QX11Info::display();
 #endif
-}
 #endif
 
+    return 0;
+}
+
 void QzTools::setWmClass(const QString &name, const QWidget* widget)
 {
 #ifdef QZ_WS_X11
@@ -910,3 +952,4 @@ QString QzTools::operatingSystem()
     return str;
 #endif
 }
+
